/**
 * Cecilia ADL Compiler
 * Copyright (C) 2006-2007 STMicroelectronics
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact: fractal@objectweb.org
 *
 * Initial Developper : J.-P. Fassino.
 * Author: Matthieu Leclercq
 * Contributor: Alessio Pace
 */

package org.objectweb.fractal.cecilia.adl.idl.remote;

import java.io.InputStream;

import org.objectweb.fractal.adl.ADLException;
import org.objectweb.fractal.cecilia.adl.idl.BasicIDLLoader;
import org.objectweb.fractal.cecilia.adl.idl.IDLErrors;
import org.objectweb.fractal.cecilia.adl.idl.ast.IDLDefinition;
import org.objectweb.fractal.cecilia.adl.idl.parser.IDLParser;
import org.objectweb.fractal.cecilia.adl.idl.remote.parser.ParseException;
import org.objectweb.fractal.cecilia.adl.idl.remote.parser.RemoteIDLParser;

public class BasicRemoteIDLLoader extends BasicIDLLoader {

  /** Override the standard DTD with the one supporting remote IDLs */
  public static final String IDL_DTD = "classpath://org/objectweb/fractal/cecilia/adl/idl/remote/ast/cecilia-remote-idl.dtd";

  /**
   * Creates an {@link IDLParser} and uses it to parse the IDL.
   * 
   * @param inputStream
   * @param srcFile
   * @return
   * @throws IDLSyntaxErrorException
   */
  @Override
  protected IDLDefinition parseIDL(final InputStream inputStream,
      final String signature, final String srcFile) throws ADLException {
    IDLDefinition node;
    final RemoteIDLParser parser = new RemoteIDLParser(inputStream);
    parser.init(nodeFactoryItf, IDL_DTD, srcFile);
    try {
      node = parser.getIDLNode();
    } catch (final ParseException e) {
      throw new ADLException(IDLErrors.SYNTAX_ERROR, signature, e.getMessage());
    }
    return node;
  }
}
