/***
 * Think ADL Parser
 * Copyright (C) 2006-2007 STMicroelectronics
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact: think@objectweb.org
 *
 * Author: Matthieu Leclercq
 */

package org.objectweb.fractal.cecilia.adl.idl.remote.ast;

import org.objectweb.fractal.cecilia.adl.idl.ast.InterfaceDefinition;

/**
 * Extends the interface of {@link InterfaceDefinition interfaceDefinition} AST
 * node to add a {@link #getRemote() remote} attribute.
 */
public interface RemoteInterfaceDefinition extends InterfaceDefinition {

  /**
   * The value of the <code>remote</code> attribute if the interface
   * represented by this node is a remote interface.
   */
  String TRUE  = "true";

  /**
   * The value of the <code>remote</code> attribute if the interface
   * represented by this node is not a remote interface.
   */
  String FALSE = "false";

  /**
   * Returns the value of the <code>remote</code> attribute of this node. If
   * {@link #TRUE "true"}, the interface represented by this node is a remote
   * interface.
   * 
   * @return the value of the <code>remote</code> attribute of this node.
   */
  String getRemote();

  /**
   * Sets the value of the <code>remote</code> attribute of this node.
   * 
   * @param remote the new value of the <code>remote</code> attribute of this
   *            node.
   */
  void setRemote(String remote);
}
