/***
 * Think ADL Parser
 * Copyright (C) 2006-2007 STMicroelectronics
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact: think@objectweb.org
 *
 * Author: Matthieu Leclercq
 */

package org.objectweb.fractal.cecilia.adl.interfaces;

import java.util.Map;

import org.objectweb.fractal.adl.ADLException;
import org.objectweb.fractal.adl.implementations.ImplementationCodeLoader;
import org.objectweb.fractal.adl.interfaces.IDLLoader;
import org.objectweb.fractal.api.NoSuchInterfaceException;
import org.objectweb.fractal.api.control.BindingController;
import org.objectweb.fractal.api.control.IllegalBindingException;
import org.objectweb.fractal.api.control.IllegalLifeCycleException;

/**
 * Load implementation code of interface adapter. This component delegates
 * loading to the {@link IDLLoader} since the implementation code of an
 * interface adapter is an IDL definition.
 */
public class InterfaceAdapterImplementationCodeLoader
    implements
      ImplementationCodeLoader,
      BindingController {

  // --------------------------------------------------------------------------
  // Client interface.
  // --------------------------------------------------------------------------

  /** The name of the {@link #idlLoaderItf} client interface. */
  public static final String IDL_LOADER_ITF_NAME = "interface-loader";

  /** The {@link IDLLoader} client interface. */
  public IDLLoader           idlLoaderItf;

  // --------------------------------------------------------------------------
  // Implementation of the ImplementationCodeLoader interface
  // --------------------------------------------------------------------------

  public Object loadImplementation(final String signature,
      final String language, final Map<Object, Object> context)
      throws ADLException {
    return idlLoaderItf.loadInterface(signature, context);
  }

  // --------------------------------------------------------------------------
  // Implementation of the BindingController interface
  // --------------------------------------------------------------------------

  public String[] listFc() {
    return new String[]{IDL_LOADER_ITF_NAME};
  }

  public void bindFc(final String clientItfName, final Object serverItf)
      throws NoSuchInterfaceException, IllegalBindingException,
      IllegalLifeCycleException {
    if (clientItfName == null) {
      throw new IllegalArgumentException("Interface name can't be null");
    }

    if (clientItfName.equals(IDL_LOADER_ITF_NAME)) {
      idlLoaderItf = (IDLLoader) serverItf;
    } else {
      throw new NoSuchInterfaceException("There is no interface named '"
          + clientItfName + "'");
    }
  }

  public Object lookupFc(final String clientItfName)
      throws NoSuchInterfaceException {
    if (clientItfName == null) {
      throw new IllegalArgumentException("Interface name can't be null");
    }

    if (clientItfName.equals(IDL_LOADER_ITF_NAME)) {
      return idlLoaderItf;
    } else {
      throw new NoSuchInterfaceException("There is no interface named '"
          + clientItfName + "'");
    }
  }

  public void unbindFc(final String clientItfName)
      throws NoSuchInterfaceException, IllegalBindingException,
      IllegalLifeCycleException {
    if (clientItfName == null) {
      throw new IllegalArgumentException("Interface name can't be null");
    }

    if (clientItfName.equals(IDL_LOADER_ITF_NAME)) {
      idlLoaderItf = null;
    } else {
      throw new NoSuchInterfaceException("There is no interface named '"
          + clientItfName + "'");
    }
  }
}
