
package org.objectweb.fractal.cecilia.primitive.cbst.implementations;

import org.objectweb.fractal.adl.components.ComponentContainer;
import org.objectweb.fractal.adl.implementations.Implementation;
import org.objectweb.fractal.api.Component;
import org.objectweb.fractal.cecilia.adl.directives.Include;
import org.objectweb.fractal.cecilia.adl.file.CodeWriter;
import org.objectweb.fractal.cecilia.primitive.thinkMC.implementations.MacroDefinitionVisitor;
import org.objectweb.fractal.task.core.TaskException;

/**
 * Visitor component that provides the inclusion code of the implementation file
 * for <code>ThinkMC</code> dialect.
 */
public class CbstMacroDefinitionVisitor extends MacroDefinitionVisitor {

  static void appendMacros(final CodeWriter cw, final String componentCName) {

    /* Define specified DECLARE_DATA macro for this component */
    cw.append("#define DECLARE_DATA \\").endl();
    cw.append("  struct ").append(componentCName).append("_instancedata")
        .endl();
    /* Define the specific METHOD macro for this component */
    cw.append("#define METHOD(i, m) \\").endl();
    cw.append(componentCName).append("##_##i##_##m##_method").endl();
    cw.append("#define CALLMINE(itf,proc,args...) \\").endl();
    cw.append("  ").append(componentCName).append(
        "##_##itf##_##proc##_method(_this, ##args)").endl();
    /* Define REQUIRED, ATTRIBUTES and DATA as a component specific macros */
    cw.append("#define REQUIRED \\").endl();
    cw.append("  ((struct ").append(componentCName).append(
        "_t *)_this)->type.imported ").endl();
    cw.append("#define ATTRIBUTES \\").endl();
    cw.append("  ((struct ").append(componentCName).append(
        "_t *)_this)->attributes ").endl();
    cw.append("#define DATA \\").endl();
    cw.append("  ((struct ").append(componentCName + "_t *)_this)->data ")
        .endl();
    /* Define the GETSELF macro according to above defined structures */
    cw.append("#define GETSELF \\").endl();
    cw.append("  struct ").append(componentCName).append(
        "_t *self = (struct " + componentCName).append("_t *) _this ").endl();

    // Define the GET_MY_INTERFACE macro
    cw.append("#define GET_MY_INTERFACE(itf) \\").endl();
    cw.append("  (&(((struct ").append(componentCName).append(
        "_t *)_this)->type.exported.itf))").endl();
  }

  // ---------------------------------------------------------------------------
  // Utility methods
  // ---------------------------------------------------------------------------

  @Override
  protected Component createImplementationTask(
      final ComponentContainer container, final Implementation impl)
      throws TaskException {
    return taskFactoryItf.newPrimitiveTask(new CbstImplementationTask(),
        container);
  }

  @Override
  protected Component createModuleImplementationTask(
      final ComponentContainer container, final Include include)
      throws TaskException {
    return taskFactoryItf.newPrimitiveTask(new CbstModuleImplementationTask(),
        container, include);
  }

  public static class CbstImplementationTask extends ImplementationTask {

    @Override
    protected String processSourceCode() throws Exception {
      final CodeWriter cw = new CodeWriter("Implementation Definition Builder");
      final String componentCName = typeNameProviderItf.getCTypeName();

      CbstMacroDefinitionVisitor.appendMacros(cw, componentCName);

      // Print the implementation code.
      appendImplementationCode(cw);
      return cw.toString();
    }
  }

  public static class CbstModuleImplementationTask
      extends
        ModuleImplementationTask {

    @Override
    protected String processSourceCode() throws Exception {
      final CodeWriter cw = new CodeWriter("Implementation Definition Builder");
      final String componentCName = typeNameProviderItf.getCTypeName();

      CbstMacroDefinitionVisitor.appendMacros(cw, componentCName);

      // Print the implementation code.
      appendImplementationCode(cw);
      return cw.toString();
    }
  }
}