/***
 * Think ADL Parser
 * Copyright (C) 2006-2007 STMicroelectronics
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact: think@objectweb.org
 *
 * Author: Matthieu Leclercq
 */

package org.objectweb.fractal.cecilia.primitive.mp;

import static org.objectweb.fractal.cecilia.adl.idl.util.Util.getPrimitiveType;

import java.util.Map;

import org.objectweb.fractal.adl.ADLException;
import org.objectweb.fractal.cecilia.adl.idl.ast.Method;
import org.objectweb.fractal.cecilia.adl.idl.ast.MethodContainer;
import org.objectweb.fractal.cecilia.adl.idl.ast.Parameter;
import org.objectweb.fractal.cecilia.adl.idl.ast.ParameterContainer;
import org.objectweb.fractal.cecilia.adl.idl.ast.PrimitiveType;
import org.objectweb.fractal.cecilia.adl.idl.ast.Type;
import org.objectweb.fractal.cecilia.adl.idl.ast.PrimitiveType.PrimitiveTypeEnum;
import org.objectweb.fractal.cecilia.adl.idl.remote.AbstractRemoteIDLLoader;
import org.objectweb.fractal.cecilia.adl.idl.remote.CSerializationHelper;
import org.objectweb.fractal.cecilia.adl.idl.remote.EXBErrors;
import org.objectweb.fractal.cecilia.adl.idl.remote.ast.RemoteInterfaceDefinition;
import org.objectweb.fractal.cecilia.adl.idl.util.Util;

/**
 * Checks that remote IDL can be handled by the Stub/Skeleton generators.
 */
public class IDLChecker extends AbstractRemoteIDLLoader {

  @Override
  protected boolean isSerializable(final Type type) {
    return CSerializationHelper.isSerializable(type);
  }

  @Override
  protected void checkRemoteInterface(
      final RemoteInterfaceDefinition definition,
      final Map<Object, Object> context) throws ADLException {

    for (final Method m : ((MethodContainer) definition).getMethods()) {
      final Type returnType = Util.getContainedType((m));

      /* The method must return void */
      final boolean isVoidMethod = (returnType instanceof PrimitiveType)
          && getPrimitiveType((PrimitiveType) returnType) == PrimitiveTypeEnum.VOID;
      if (!isVoidMethod)
        throw new ADLException(EXBErrors.IDL_INVALID_TYPE,
            "Each method in MP stubs and skeletons must return void");
      ;

      /* Each parameter must be serializable and must not be in or in/out */
      for (final Parameter p : ((ParameterContainer) m).getParameters()) {
        final Type paramType = Util.getContainedType((p));
        if (!isSerializable(paramType))
          throw new ADLException(EXBErrors.IDL_INVALID_TYPE,
              "The type of the parameter '" + p.getName()
                  + "' is not serializable", paramType);
        if (Util.isOutParameter(p) || Util.isInOutParameter(p))
          throw new ADLException(EXBErrors.IDL_INVALID_TYPE, "The parameter '"
              + p.getName() + "' must be IN", paramType);
      }
    }
  }
}
